from importlib.util import module_from_spec, spec_from_file_location
from traceback import format_exc, print_exc
from ujson import loads, dumps, JSONDecodeError # type: ignore
from shutil import copyfileobj
from gzip import open as gzipopen
from pyrogram.filters import create

from sys import exit
from os import getcwd, makedirs, stat, walk
from os import path
from time import localtime, strftime, time
from datetime import datetime

from modules.colors import *

#=================================================================================
def getTime(timestamp):
    if isinstance(timestamp, datetime):
        timestamp = timestamp.timestamp()
    return strftime("%H:%M:%S", localtime(int(timestamp)))

def getDateTime(timestamp):
    if isinstance(timestamp, datetime):
        timestamp = timestamp.timestamp()
    return strftime("%H:%M:%S | %d.%m.%Y", localtime(int(timestamp)))

def getChatName(message, index=False):
    if message.chat.title == None:
        if index is False:
            return "DM"
        else:
            title = ""
            if message.chat.first_name != None: title += message.chat.first_name
            if message.chat.last_name != None: title += " " + message.chat.last_name
            return title
    else:
        return message.chat.title
#=================================================================================

# Get data from json file
def jsonLoad(filename):
    with open(filename, 'r', encoding="utf-8") as json_file:
        output = loads(json_file.read())
        json_file.close()
    return output

# Save data to json file
def jsonSave(filename, value):
    with open(filename, 'w', encoding="utf-8") as f:
        f.write(dumps(value, indent=4, ensure_ascii=False))
        f.close()

# Get config key's value
def configGet(key: str):
    try:
        return jsonLoad("config.json")[key]
    except FileNotFoundError:
        print("Could not find config.json in script's directory. Please copy config_example.json to config.json and configure it. After that restart this program.")
        exit()


def indexId(chatid, title):

    makedirs("data", exist_ok=True)

    try:
        index = jsonLoad("data/index.json")
    except FileNotFoundError:
        index = {}
    except JSONDecodeError:
        index = {}

    if str(chatid) in index:
        if str(title) in index[str(chatid)]:
            return
        else:
            index[str(chatid)][str(title)] = time()
    else:
        index[str(chatid)] = {
            str(title): time()
        }

    jsonSave("data/index.json", index)


# Check latest log size
def checkSize(file="latest"):

    try:

        if file == "latest":
            logfile = 'logs/latest.log'
            logzipped = f'logs/{datetime.now().strftime("%d.%m.%Y_%H:%M:%S")}.log.gz'
        elif file == "errors":
            logfile = 'logs/errors.log'
            logzipped = f'logs/errors_{datetime.now().strftime("%d.%m.%Y_%H:%M:%S")}.log.gz'
        else:
            logfile = f'logs/chats/{file}/latest.log'
            logzipped = f'logs/chats/{file}/{datetime.now().strftime("%d.%m.%Y_%H:%M:%S")}.log.gz'

        log = stat(logfile)

        if (log.st_size / 1024) > configGet("log_size"):
            with open(logfile, 'rb') as f_in:
                with gzipopen(logzipped, 'wb') as f_out:
                    copyfileobj(f_in, f_out)
                    print(logzipped)
            open(logfile, 'w').close()

    except FileNotFoundError:
        if configGet("debug"):
            print_exc()
            print("Latest log file does not exist", flush=True)
        pass


# Append to log file
def appendLog(message, file="latest"):

    makedirs("logs", exist_ok=True)
    makedirs("logs/chats", exist_ok=True)

    if file == "latest":
        checkSize()
        log = open('logs/latest.log', 'a')
    elif file == "errors":
        checkSize(file)
        log = open('logs/errors.log', 'a')
    else:
        makedirs(f"logs/chats/{file}", exist_ok=True)
        checkSize(file)
        log = open(f'logs/chats/{file}/latest.log', 'a')

    log.write(f'{message}\n')
    log.close()

# Command match function
def match(type, firstname, text, msg):

    try:
        low_txt = text.lower()
        usr_fn = firstname
    except:
        low_txt = None
        usr_fn = None

    if low_txt != None and usr_fn != None:
        print(f'{BBLACK}[{CYAN}{getTime(msg.date)}{BBLACK}] [{YELLOW}{type}{BBLACK}] {RESET}{usr_fn} ({getChatName(msg)}) {BBLACK}» {RESET}{low_txt}')
        appendLog(f'[{getDateTime(msg.date)}] [{type}] {usr_fn} ({getChatName(msg)}) » {low_txt}', file=msg.chat.id)

def getRequired(msg, mode):
    
    from_name = msg.from_user.first_name

    try:
        if mode == "Image":
            msg.reply_photo(msg.text[5:], quote=False)
        elif mode == "Document":
            msg.reply_document(msg.text[5:], quote=False)
        elif mode == "Video":
            msg.reply_video(msg.text[5:], quote=False)
        elif mode == "Animation":
            msg.reply_animation(msg.text[5:], quote=False)
        elif mode == "Voice":
            msg.reply_voice(msg.text[5:], quote=False)
        elif mode == "Audio":
            msg.reply_audio(msg.text[5:], quote=False)
        elif mode == "VideoNote":
            msg.reply_video_note(msg.text[5:], quote=False)
        elif mode == "Sticker":
            msg.reply_sticker(msg.text[5:], quote=False)
        else:
            pass

        print(f'{BBLACK} [{CYAN}{getTime(msg.date)}{BBLACK}] [{YELLOW}MATCH{BBLACK}] {RESET}{from_name} ({getChatName(msg)}) {BBLACK}» {RESET}{msg.text}')
        appendLog(f'[{getDateTime(msg.date)}][MATCH]  {from_name} ({getChatName(msg)}) » {msg.text}')
    except Exception as excep:
        if msg.from_user.id == configGet("ownerid"):
            if configGet("debug"):
                msg.edit('Не удалось получить ничего по этому коду/адресу.\n\n```'+str(excep)+'```')
            else:
                msg.edit('Не удалось получить ничего по этому коду/адресу.')
            print(f'{BBLACK} [{CYAN}{getTime(msg.date)}{BBLACK}] [{YELLOW}MATCH{BBLACK}] {RESET}{from_name} ({getChatName(msg)}) {BBLACK}» {RESET}{msg.text} {BBLACK}({BRED}FAILED{BBLACK}){RESET}')
            appendLog(f'[{getDateTime(msg.date)}] [MATCH] {from_name} ({getChatName(msg)}) » {msg.text} (FAILED)')
        else:
            msg.reply_text('Не удалось получить ничего по этому коду/адресу.')
            print(f'{BBLACK} [{CYAN}{getTime(msg.date)}{BBLACK}] [{YELLOW}MATCH{BBLACK}] {RESET}{from_name} ({getChatName(msg)}) {BBLACK}» {RESET}{msg.text} {BBLACK}({BRED}FAILED{BBLACK}){RESET}')
            appendLog(f'[{getDateTime(msg.date)}] [MATCH] {from_name} ({getChatName(msg)}) » {msg.text} (FAILED)')

#=================================================================================

# Import functions
# Took from https://stackoverflow.com/a/57892961
def get_py_files(src):
    cwd = getcwd() # Current Working directory
    py_files = [] 
    for root, dirs, files in walk(src):
        for file in files:
            if file.endswith(".py"):
                py_files.append(path.join(cwd, root, file))
    return py_files


def dynamic_import(module_name, py_path):
    try:
        module_spec = spec_from_file_location(module_name, py_path)
        module = module_from_spec(module_spec)
        module_spec.loader.exec_module(module)
        return module
    except SyntaxError:
        print(f"{BBLACK}[{CYAN}{getTime(time())}{BBLACK}] {RESET}Could not load extension {BGREEN}{module_name} {RESET}due to {BRED}invalid syntax{RESET}. Check {BYELLOW}logs/errors.log {RESET}for details.", flush=True)
        appendLog(f"[{getDateTime(time())}] Could not import module {module_name}. Here's traceback:\n{format_exc()}", file="errors")
        return
    except Exception as exp:
        print(f"{BBLACK}[{CYAN}{getTime(time())}{BBLACK}] {RESET}Could not load extension {BGREEN}{module_name} {RESET}due to {BRED}{exp}{RESET}", flush=True)
        return


def dynamic_import_from_src(src, star_import = False):
    my_py_files = get_py_files(src)
    for py_file in my_py_files:
        module_name = path.split(py_file)[-1][:-3]
        print(f"{BBLACK}[{CYAN}{getTime(time())}{BBLACK}] {RESET}Importing {BGREEN}{module_name} {RESET}extension...", flush=True)
        imported_module = dynamic_import(module_name, py_file)
        if imported_module != None:
            if star_import:
                for obj in dir(imported_module):
                    globals()[obj] = imported_module.__dict__[obj]
            else:
                globals()[module_name] = imported_module
        print(f"{BBLACK}[{CYAN}{getTime(time())}{BBLACK}] {RESET}Successfully loaded {BGREEN}{module_name} {RESET}extension", flush=True)
    return

#=================================================================================

async def blacklisted_func(_, __, query):
    if query.chat.id in configGet("ignore_ids"):
        return True
    else:
        return False
    

blacklisted = create(blacklisted_func)